@@ -15,6 +15,8 @@
 
 """Tests for the notifications api."""
 
+import copy
+
 import ddt
 import mock
 from oslo_serialization import jsonutils
@@ -27,6 +29,7 @@ from masakari.engine import rpcapi as engine_rpcapi
 from masakari import exception
 from masakari.ha import api as ha_api
 from masakari.objects import base as obj_base
+from masakari.objects import fields
 from masakari.objects import notification as notification_obj
 from masakari import test
 from masakari.tests.unit.api.openstack import fakes
@@ -34,12 +37,17 @@ from masakari.tests.unit.objects import test_objects
 from masakari.tests import uuidsentinel
 
 NOW = timeutils.utcnow().replace(microsecond=0)
+OPTIONAL = ['recovery_workflow_details']
 
 
 def _make_notification_obj(notification_dict):
     return notification_obj.Notification(**notification_dict)
 
 
+def _make_notification_progress_details_obj(progress_details):
+    return notification_obj.NotificationProgressDetails(**progress_details)
+
+
 def _make_notifications_list(notifications_list):
     return notification_obj.Notification(objects=[
         _make_notification_obj(a) for a in notifications_list])
@@ -60,6 +68,22 @@ NOTIFICATION_DATA = {"type": "VM", "id": 1,
 
 NOTIFICATION = _make_notification_obj(NOTIFICATION_DATA)
 
+RECOVERY_DETAILS = {"progress": 1.0,
+                    "state": "SUCCESS",
+                    "name": "StopInstanceTask",
+                    "progress_details": [
+                        {"timestamp": "2019-03-07 13:54:28",
+                         "message": "Stopping instance",
+                         "progress": "0.0"},
+                    ]}
+
+NOTI_DATA_WITH_DETAILS = copy.deepcopy(NOTIFICATION_DATA)
+
+NOTIFICATION_WITH_PROGRESS_DETAILS = _make_notification_obj(
+    NOTI_DATA_WITH_DETAILS)
+RECOVERY_OBJ = _make_notification_progress_details_obj(RECOVERY_DETAILS)
+NOTIFICATION_WITH_PROGRESS_DETAILS.recovery_workflow_details = [RECOVERY_OBJ]
+
 NOTIFICATION_LIST = [
     {"type": "VM", "id": 1, "payload": {'event': 'STOPPED',
                                         'host_status': 'NORMAL',
@@ -154,14 +178,34 @@ class NotificationTestCase(test.TestCase):
 
         mock_create.return_value = NOTIFICATION
         result = self.controller.create(self.req, body={
-            "notification": {"hostname": "fake_host",
-                             "payload": {"event": "STOPPED",
-                                         "host_status": "NORMAL",
-                                         "cluster_status": "ONLINE"},
-                             "type": "VM",
-                             "generated_time": "2016-09-13T09:11:21.656788"}})
+            "notification": {
+                "hostname": "fake_host",
+                "payload": {
+                    "instance_uuid": uuidsentinel.instance_uuid,
+                    "vir_domain_event": "STOPPED_FAILED",
+                    "event": "LIFECYCLE"
+                },
+                "type": "VM",
+                "generated_time": "2016-09-13T09:11:21.656788"}})
         result = result['notification']
-        test_objects.compare_obj(self, result, NOTIFICATION_DATA)
+        test_objects.compare_obj(self, result, NOTIFICATION_DATA,
+                                 allow_missing=OPTIONAL)
+
+    @mock.patch.object(ha_api.NotificationAPI, 'create_notification')
+    def test_create_process_notification(self, mock_create):
+        mock_create.return_value = NOTIFICATION
+        result = self.controller.create(self.req, body={
+            "notification": {
+                "hostname": "fake_host",
+                "payload": {
+                    "process_name": "nova-compute",
+                    "event": "STOPPED"
+                },
+                "type": "PROCESS",
+                "generated_time": "2016-09-13T09:11:21.656788"}})
+        result = result['notification']
+        test_objects.compare_obj(self, result, NOTIFICATION_DATA,
+                                 allow_missing=OPTIONAL)
 
     @mock.patch('masakari.rpc.get_client')
     @mock.patch.object(ha_api.NotificationAPI, 'create_notification')
@@ -171,9 +215,9 @@ class NotificationTestCase(test.TestCase):
             "notification": {
                 "hostname": "fake_host",
                 "payload": {
-                    "event": "STOPPED",
-                    "host_status": "NORMAL",
-                    "cluster_status": "ONLINE"
+                    "instance_uuid": uuidsentinel.instance_uuid,
+                    "vir_domain_event": "STOPPED_FAILED",
+                    "event": "LIFECYCLE"
                 },
                 "type": "VM",
                 "generated_time": NOW
@@ -189,12 +233,17 @@ class NotificationTestCase(test.TestCase):
     @mock.patch.object(ha_api.NotificationAPI, 'create_notification')
     def test_create_host_not_found(self, mock_create):
         body = {
-            "notification": {"hostname": "fake_host",
-                             "payload": {"event": "STOPPED",
-                                         "host_status": "NORMAL",
-                                         "cluster_status": "ONLINE"},
-                             "type": "VM",
-                             "generated_time": "2016-09-13T09:11:21.656788"}}
+            "notification": {
+                "hostname": "fake_host",
+                "payload": {
+                    "instance_uuid": uuidsentinel.instance_uuid,
+                    "vir_domain_event": "STOPPED_FAILED",
+                    "event": "LIFECYCLE"
+                },
+                "type": "VM",
+                "generated_time": "2016-09-13T09:11:21.656788"
+            }
+        }
         mock_create.side_effect = exception.HostNotFoundByName(
             host_name="fake_host")
         self.assertRaises(exc.HTTPBadRequest, self.controller.create,
@@ -272,6 +321,54 @@ class NotificationTestCase(test.TestCase):
         self.assertRaises(self.bad_request, self.controller.create,
                           self.req, body=body)
 
+    @ddt.data(
+        # invalid event for PROCESS type
+        {"params": {"payload": {"event": "invalid",
+                                "process_name": "nova-compute"},
+                    "type": fields.NotificationType.PROCESS}},
+
+        # invalid event for VM type
+        {"params": {"payload": {"event": "invalid",
+                                "host_status": fields.HostStatusType.NORMAL,
+                          "cluster_status": fields.ClusterStatusType.ONLINE},
+                    "type": fields.NotificationType.VM}},
+
+        # invalid event for HOST_COMPUTE type
+        {"params": {"payload": {"event": "invalid"},
+                    "type": fields.NotificationType.COMPUTE_HOST}},
+
+        # empty payload
+        {"params": {"payload": {},
+                    "type": fields.NotificationType.COMPUTE_HOST}},
+
+        # empty process_name
+        {"params": {"payload": {"event": fields.EventType.STOPPED,
+                                "process_name": ""},
+                    "type": fields.NotificationType.PROCESS}},
+
+        # process_name too long value
+        {"params": {"payload": {"event": fields.EventType.STOPPED,
+                              "process_name": "a" * 4097},
+                  "type": fields.NotificationType.PROCESS}},
+
+        # process_name invalid data_type
+        {"params": {"payload": {"event": fields.EventType.STOPPED,
+                              "process_name": 123},
+                  "type": fields.NotificationType.PROCESS}}
+    )
+    @ddt.unpack
+    def test_create_with_invalid_payload(self, params):
+        body = {
+            "notification": {"hostname": "fake_host",
+                             "generated_time": "2016-09-13T09:11:21.656788"
+                             }
+        }
+
+        body['notification']['payload'] = params['payload']
+        body['notification']['type'] = params['type']
+        self.assertRaises(self.bad_request, self.controller.create,
+                          self.req, body=body)
+
     @mock.patch.object(ha_api.NotificationAPI, 'create_notification')
     def test_create_duplicate_notification(self, mock_create_notification):
         mock_create_notification.side_effect = exception.DuplicateNotification(
@@ -374,3 +471,30 @@ class NotificationCasePolicyNotAuthorized(test.NoDBTestCase):
                                 self.controller.index,
                                 self.req)
         self._check_rule(exc, rule_name)
+
+
+class NotificationV1_1_TestCase(NotificationTestCase):
+    """Test Case for notifications api for 1.1 API"""
+    api_version = '1.1'
+
+    @mock.patch.object(engine_rpcapi, 'EngineAPI')
+    def setUp(self, mock_rpc):
+        super(NotificationV1_1_TestCase, self).setUp()
+        self.controller = notifications.NotificationsController()
+        self.req = fakes.HTTPRequest.blank('/v1/notifications',
+                                           use_admin_context=True,
+                                           version=self.api_version)
+        self.context = self.req.environ['masakari.context']
+
+    @mock.patch.object(ha_api.NotificationAPI,
+                       'get_notification_recovery_workflow_details')
+    def test_show(self, mock_get_notification_recovery_workflow_details):
+        (mock_get_notification_recovery_workflow_details
+         .return_value) = NOTIFICATION_WITH_PROGRESS_DETAILS
+
+        result = self.controller.show(self.req, uuidsentinel.fake_notification)
+        result = result['notification']
+        self.assertItemsEqual([RECOVERY_OBJ],
+                              result.recovery_workflow_details)
+        self._assert_notification_data(NOTIFICATION_WITH_PROGRESS_DETAILS,
+                                       _make_notification_obj(result))
